# Script 2. Second script to aggregate CDL classes over multiple years.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = 'Steven Lennartz'

# Import Site Packages
import arcpy
from arcpy import env
from arcpy.sa import *
import datetime

# Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

# Set variables
# In-workspace is the out-workspace from step 1
inws = r".../CDL/CDL_Recodes/180524/CDL_GenAggregates_1317_180524.gdb/"
# Create a File Geodatabase for an out-workspace
outws = r".../CDL/CDL_Recodes/180524/CDL_GenClass13_1317_180524.gdb/"
# The cultivated layer from the CDL website
cult = Raster(".../CDL/Cultivated_Layer/2017_Cultivated_Layer/2017_Cultivated_Layer.img")
# From and to years of CDL. This example is 5 years, 2013-2017, "CDL_1317_". For use in formatting strings.
cdl = "CDL_1317_"

# Environmental settings
arcpy.env.workspace = inws
arcpy.env.overwriteOutput = 1
arcpy.env.snapRaster = cult
arcpy.env.extent = cult
arcpy.env.scratchWorkspace = outws
arcpy.env.parallelProcessingFactor = "100%"

start_t1 = datetime.datetime.now()

# Conditional statements producing 13 general CDL layers
start_t = datetime.datetime.now()
print "Recoding Corn at {}".format(start_t)
out1 = Con(((Raster("{}10".format(cdl)) == 1) | (Raster("{}14".format(cdl)) == 1) | (Raster("{}15".format(cdl)) == 1)
            | (Raster("{}18".format(cdl)) == 1)), 1, 0)
out1.save("{}{}10".format(outws, cdl))
print "Building pyramids........"
arcpy.BuildPyramids_management("{}{}10".format(outws, cdl))
print "Completed Corn recode in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Recoding Cotton at {}".format(start_t)
out2 = Con(((Raster("{}20".format(cdl)) == 1) | (Raster("{}25".format(cdl)) == 1) | (Raster("{}26".format(cdl)) == 1)
            | (Raster("{}42".format(cdl)) == 1)), 1, 0)
out2.save("{}{}20".format(outws, cdl))
print "Building pyramids........"
arcpy.BuildPyramids_management("{}{}20".format(outws, cdl))
print "Completed Cotton recode in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Copying Rice at {}".format(start_t)
arcpy.Copy_management("{}30".format(cdl), "{}{}30".format(outws, cdl))
print "Completed Rice copy in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Recoding Soybeans at {}".format(start_t)
out4 = Con(((Raster("{}40".format(cdl)) == 1) | (Raster("{}42".format(cdl)) == 1) | (Raster("{}45".format(cdl)) == 1)
            | (Raster("{}48".format(cdl)) == 1) | (Raster("{}14".format(cdl)) == 1)), 1, 0)
out4.save("{}{}40".format(outws, cdl))
print "Building pyramids........"
arcpy.BuildPyramids_management("{}{}40".format(outws, cdl))
print "Completed Soybeans recode in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Recoding Wheat at {}".format(start_t)
out5 = Con(((Raster("{}50".format(cdl)) == 1) | (Raster("{}56".format(cdl)) == 1) | (Raster("{}58".format(cdl)) == 1)
            | (Raster("{}15".format(cdl)) == 1) | (Raster("{}25".format(cdl)) == 1) | (Raster("{}45".format(cdl)) ==
                                                                                       1)), 1, 0)
out5.save("{}{}50".format(outws, cdl))
print "Building pyramids........"
arcpy.BuildPyramids_management("{}{}50".format(outws, cdl))
print "Completed Wheat recode in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Recoding Veg & Ground Fruit at {}".format(start_t)
out6 = Con(((Raster("{}60".format(cdl)) == 1) | (Raster("{}61".format(cdl)) == 1) | (Raster("{}68".format(cdl)) == 1)
            | (Raster("{}26".format(cdl)) == 1) | (Raster("{}56".format(cdl)) == 1)), 1, 0)
out6.save("{}{}60".format(outws, cdl))
print "Building pyramids........"
arcpy.BuildPyramids_management("{}{}60".format(outws, cdl))
print "Completed Veg & Ground Fruit recode in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Copying Other Orchards at {}".format(start_t)
arcpy.Copy_management("{}70".format(cdl), "{}{}70".format(outws, cdl))
print "Completed Other Orchards copy in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Copying Grapes at {}".format(start_t)
arcpy.Copy_management("{}71".format(cdl), "{}{}71".format(outws, cdl))
print "Completed Grapes copy in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Copying Citrus at {}".format(start_t)
arcpy.Copy_management("{}72".format(cdl), "{}{}72".format(outws, cdl))
print "Completed Citrus copy in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Recoding Other Grains at {}".format(start_t)
out8 = Con(((Raster("{}80".format(cdl)) == 1) | (Raster("{}18".format(cdl)) == 1) | (Raster("{}48".format(cdl)) == 1)
            | (Raster("{}58".format(cdl)) == 1)), 1, 0)
out8.save("{}{}80".format(outws, cdl))
print "Building pyramids........"
arcpy.BuildPyramids_management("{}{}80".format(outws, cdl))
print "Completed Other Grains recode in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Copying Other Row Crops at {}".format(start_t)
arcpy.Copy_management("{}90".format(cdl), "{}{}90".format(outws, cdl))
print "Completed Other Row Crops copy in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Copying Other Crops at {}".format(start_t)
arcpy.Copy_management("{}100".format(cdl), "{}{}100".format(outws, cdl))
print "Completed Other Crops copy in: {}".format(datetime.datetime.now() - start_t)

start_t = datetime.datetime.now()
print "Copying Pasture, Hay, Silage at {}".format(start_t)
arcpy.Copy_management("{}110".format(cdl), "{}{}110".format(outws, cdl))
print "Completed Pasture, Hay, Silage copy in: {}".format(datetime.datetime.now() - start_t)

print "Copying Key....."
arcpy.Copy_management(".../CDL/CDL_Recodes/180424/CDL_GenClass13_1017_180424.gdb/CDL_GenClass13_key",
                      "{}CDL_GenClass13_key".format(outws))

print "Completed 13 Class Recode in: {}".format(datetime.datetime.now() - start_t1)
