# Script 5. This script populates a geodatabase with CDL Cultivated Mask rasters by county. These serve as a mask for
# the expansion script.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = 'Steven Lennartz'

# Import Site Packages
import arcpy
import datetime
from arcpy import env
from arcpy.sa import *

# Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

# Set variables
# Output directory same as cultivated layer
outdir = r".../CDL/Cultivated_Layer/2017_Cultivated_Layer/"
# Tiger Line county boundaries, vector
cnty = r".../CDL/Boundaries/St_Cnty_Boundaries.gdb/tl_2013_cnty_conus_alb"
# Unique ID for counties, "FIPS code"
fipscode = "GEOID"
# The cultivated layer from the CDL website
cult = Raster("{}2017_Cultivated_Layer.img".format(outdir))

start_t = datetime.datetime.now()

# Environmental Settings
arcpy.env.overwriteOutput = 1
arcpy.env.snapRaster = cult
arcpy.env.parallelProcessingFactor = "100%"

print "Starting process at: {}".format(start_t)

# List county FIPS codes
uniqueFips = {(row[0], row[1]) for row in arcpy.da.SearchCursor(cnty, (fipscode, "SHAPE@"))}

# Check for output geodatabase
if not arcpy.Exists("{}2017_Cultivated_Layer.gdb".format(outdir)):
    print "Creating 2017_Cultivated_Layer.gdb"
    arcpy.CreateFileGDB_management(outdir, "2017_Cultivated_Layer.gdb")
if not arcpy.Exists("{}2017_Cultivated_Layer.gdb/cultmask_2017".format(outdir)):
    # Recode CDL Cultivated Layer
    start_t2 = datetime.datetime.now()
    print "Recoding 2017_Cultivated_Layer.img"
    arcpy.env.workspace = "{}2017_Cultivated_Layer.gdb".format(outdir)
    arcpy.env.scratchWorkspace = "{}2017_Cultivated_Layer.gdb".format(outdir)
    arcpy.gp.Reclassify_sa(cult, "Value", "0 0;1 0;2 1;3 0", "{}2017_Cultivated_Layer.gdb/cultmask_2017".format(
        outdir), "DATA")
    print "Completed Recode in: {}".format(datetime.datetime.now() - start_t2)
# Cast variable for agricultural mask
agmask = Raster("{}2017_Cultivated_Layer.gdb/cultmask_2017".format(outdir))
print "Building Pyramids....."
arcpy.BuildPyramids_management(agmask)

# Clip CDL cultivated raster by county FIPS code
if not arcpy.Exists("{}2017_Cult_Cnty.gdb".format(outdir)):
    arcpy.CreateFileGDB_management(outdir, "2017_Cult_Cnty.gdb")

# Set environments
arcpy.env.workspace = "{}2017_Cult_Cnty.gdb".format(outdir)
arcpy.env.scratchWorkspace = "{}2017_Cult_Cnty.gdb".format(outdir)
arcpy.env.extent = "MAXOF"

# Make County Feature Layer
clipfc = "clip_lyr"
arcpy.MakeFeatureLayer_management(cnty, clipfc)

# Loop through counties to clip CDL cultivated layer
for fips, shape in uniqueFips:
    start_t1 = datetime.datetime.now()
    extent = shape.extent
    extent = "{} {} {} {}".format(extent.XMin, extent.YMin, extent.XMax, extent.YMax)
    arcpy.SelectLayerByAttribute_management(clipfc, "NEW_SELECTION", "{} = '{}'".format(fipscode, fips))
    out_file = "{}2017_Cult_Cnty.gdb/Cult_2017_{}".format(outdir, fips)
    print "Clipping Cult_2017_{}....".format(fips)
    arcpy.Clip_management(agmask, extent, out_file, clipfc, "2147483647", "ClippingGeometry", "NO_MAINTAIN_EXTENT")
    print "Finished clip in: {}".format(datetime.datetime.now() - start_t1)
print "Completed All County Clipping in: {}".format(datetime.datetime.now() - start_t)
