## Script 9. Create Eulclidean Distance rasters using outputs from script 8.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = ' Steven Lennartz'

## Import Site Packages
import arcpy
import datetime
from arcpy import env
from arcpy.sa import *

## Set Variables
## In-workspace is the out-workspace from script 8
inws = r".../CDL/Expand/180524/1317/Mosaic/Final/CDL1317x_mosaic2.gdb/"
path = r".../CDL/Euclidean/180524/1317/"

## Environmental settings
arcpy.env.workspace = inws
arcpy.env.overwriteOutput = 1
arcpy.env.snapRaster = Raster(".../CDL/Cultivated_Layer/2017_Cultivated_Layer/2017_Cultivated_Layer.img")
arcpy.env.parallelProcessingFactor = "100%"

## Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

start_t = datetime.datetime.now()
print "Begun processing at: {}".format(start_t)

## check for existing geodatabase
if not arcpy.Exists("{}CDL1317x_mosaic2_euc.gdb".format(path)):
    print "Creating CDL1317x_mosaic2_euc.gdb"
    arcpy.CreateFileGDB_management(path, "CDL1317x_mosaic2_euc.gdb")

## Set out-workspace
outws = r"{}CDL1317x_mosaic2_euc.gdb/".format(path)
arcpy.env.scratchWorkspace = outws

## List rasters in workspace
rasterList = arcpy.ListRasters()
## Loop through rasters to run Euclidean Distance
for raster in rasterList:
    start_t1 = datetime.datetime.now()
    print "Processing {} at {}".format(raster, datetime.datetime.now())
    inras = Raster(raster)
    desc = arcpy.Describe(inras)
    name = desc.name
    val = name.split('_')[2]
    val = val[:-2]
    ## While previous map algebra steps required "0" values, euclidean distance requires these SetNull
    print "Setting Null Values"
    out = SetNull(inras, inras, "VALUE = 0")
    ## Euclidean distance with 1,500 meter max buffer, 30 meter intervals (pixel resolution)
    out2 = EucDistance(out, "1500", "30")
    ## Setting the output to an integer greatly reduces file size
    print "Converting to integer"
    out3 = Int(out2)
    out3.save("{}CDL_1317_{}x2_euc".format(outws, val))
    print "Building Pyramids....."
    arcpy.BuildPyramids_management("{}CDL_1317_{}x2_euc".format(outws, val))
    print "Completed Euclidean Distance for Class {} in: {}".format(val, datetime.datetime.now() - start_t1)
print "Completed Euclidean Distance for all classes in: {}".format(datetime.datetime.now() - start_t)
